program testL2

  use sep
  use prop_mod1D

  use solver_smp_mod
  use cgstep_mod

  implicit none

  integer                                      ::  niter         ! number of iterations
  logical(4) :: success
  logical                               :: wfldmovie, DW, MW, sder
  real                                  :: dx, dz, oz, ost, dst, ot_data, dt_data, dt_prop
  real                                  :: zsrc, zrec,vmax,tdelay,tapeco
  real                                  :: dot1, dot2
  integer                               :: nnz, nz, nst, nt_data, nop, npad
  integer            			:: iz , it, orderT, bout
  real, allocatable, dimension(:)     :: vel, mwarr, dat, DWarray
  real, allocatable, dimension(:)     :: image, resdArr
  real, allocatable, dimension(:)       ::wavelet, mwarr1D, waveletVD
  real, allocatable, dimension(:)       ::x1, x2, y1, y2
real, allocatable, dimension(:,:)       :: mmov
  character(len=128)                    :: souWfile_, imgWfile_, recWfile_
  character(len=128)                    :: outfile_, outmov_

  call sep_init()

  ! read in velocity model
  call from_history("n1", nz)
  call from_history("o1", oz)
  call from_history("d1", dz)

  ! read source wavelet information
  call from_aux("aux", "n1", nst)
  call from_aux("aux", "o1", ost)
  call from_aux("aux", "d1", dst)

  ! read shot gather (data) file info
  call from_aux("datafile", "n1", nt_data)
  call from_aux("datafile", "o1", ot_data)
  call from_aux("datafile", "d1", dt_data)

  write(0,*) 'nt_data, ot, dt', nt_data, ot_data, dt_data

  ! read source location
  call from_param("zsrc",zsrc)
  call from_param("npad",npad,30)
  call from_param("zrec",zrec,zsrc)
  
  ! Read wavefield movie switch
  call from_param("wfldout",wfldmovie,.false.)
  call from_param("DW",DW,.false.);   call from_param("MW",MW,.false.)
  call from_param("souwfld",souWfile_,"souwfld.H")
  call from_param("imgwfld",imgWfile_,"imgwfld.H")  
  call from_param("recwfld",recWfile_,"recwfld.H")
  call from_param("outfile",outfile_)
  call from_param("tdelay",tdelay)
  call from_param("dt_prop",dt_prop,0.001)
  call from_param("tapeco",tapeco,0.00002);
  call from_param("orderT",orderT,2);
  call from_param("sder", sder, .false.);
  
  write(0,*) 'nt_data, dt_data, ot_data', nt_data, dt_data, ot_data
  write(0,*) 'dt_prop', dt_prop  


  if (dst .ne. dt_data) call erexit('dst should equal to dt_data')
  if (dt_prop .ne. dt_data) call erexit('expect dt_prop .eq. dt_data for now')

  nnz=nz
  nz = nnz+2*npad

  ! Allocate memory
  write(0,*) 'memory', nt_data, nnz, nz
  allocate(vel(nz),image(nnz),wavelet(nt_data), waveletVD(nt_data))
  allocate(dat(nt_data),resdArr(nt_data))
  allocate(mmov(nnz,niter))
  vel=0.
  image=0.
  wavelet=0.
  waveletVD=0.
  dat=0.
  resdArr=0.
  mmov=0.

  ! Read velocity model
  call sep_read(vel(npad+1:nz-npad));  call from_param("vmax",vmax);
  ! padding
  do iz=1,npad
     vel(iz) = vel(npad+1)
     vel(nz-iz+1) = vel(nz-npad-1)
  end do

  ! Read source wavelet
  call sep_read(wavelet,"aux")
    call Ver2ndDer(.true.,.false.,wavelet,waveletVD,nt_data)
  
  ! Read data
  call sep_read(dat,'datafile')


  call from_param("outfile",outfile_,'outfile.H')

  call from_param ("niter", niter, 2)
  write(0,*) "niter   =",niter
  write(0,*) "orderT", orderT

  ! Initiate propagation
  call init_propagate(vel,oz,dz,dst,dt_prop,nt_data,dt_data,ot_data, tdelay,vmax,npad,tapeco,orderT, wfldmovie,souWfile_, recWfile_, imgWfile_) 
  ! Initiate source parameters
  if (sder) then
     call sou_rec_init(waveletVD,zsrc,zrec,tdelay)
  else
    call sou_rec_init(wavelet,zsrc,zrec,tdelay)
  end if


  ! Run the inversion over niter-ations
  call solver_smp( m=image,d=dat, Fop=born2way_op  , stepper=cgstep,niter=niter, resd= resdArr, verb=.true.)


   call close_propagate()
   call sou_rec_close()

   call to_history("n1",nnz,outfile_)
    call to_history("o1",oz,outfile_)
    call to_history("d1",dz,outfile_)
    call to_history  ("label1","z(m)",outfile_)
    call sep_write(image,outfile_)

  outmov_ ='image_movie.H'

   if (.false.) then
    call auxputch("n1",'i',nnz,outmov_)
    call auxputch("o1",'f',oz,outmov_)
    call auxputch("d1",'f',dz,outmov_)
    call auxputch("label1",'s',"z(m)",outmov_)
    call auxputch("n2",'i',niter,outmov_)
    call auxputch("o2",'f',1.,outmov_)
    call auxputch("d2",'f',1.,outmov_)
    call auxputch("label2",'s',"iter",outmov_)
    call sep_write(mmov,outmov_)
  end if




  deallocate (image, dat)




end program
